const express = require('express');
const multer = require('multer');
const { S3Client, PutObjectCommand } = require('@aws-sdk/client-s3');
const { DynamoDBDocumentClient, PutCommand, QueryCommand } = require('@aws-sdk/lib-dynamodb');
const { v4: uuidv4 } = require('uuid');
const router = express.Router();

// Configuration AWS
const dbClient = new DynamoDBClient({ region: 'us-east-1' });
const ddb = DynamoDBDocumentClient.from(dbClient);
const s3 = new S3Client({ region: 'us-east-1' });

const IMAGES_TABLE = 'Images';
const BUCKET_NAME = 'tp1aws-1';

const upload = multer();

// Page d'accueil avec l'album de l'utilisateur
router.get('/home', async (req, res) => {
    const { username } = req.query;

    if (!username) {
        return res.redirect('/');
    }

    try {
        // Récupérer les images de l'utilisateur
        const data = await ddb.send(new QueryCommand({
            TableName: IMAGES_TABLE,
            KeyConditionExpression: 'username = :username',
            ExpressionAttributeValues: {
                ':username': username,
            },
        }));

        res.render('home', {
            username,
            images: data.Items || [],
        });
    } catch (err) {
        console.error(err);
        res.render('home', {
            username,
            images: [],
            error: 'Erreur lors de la récupération des images',
        });
    }
});

// Route pour uploader une image
router.post('/upload', upload.single('file'), async (req, res) => {
    const { username } = req.body;
    const file = req.file;

    if (!username || !file) {
        return res.status(400).json({ error: 'Nom d\'utilisateur ou fichier manquant' });
    }

    const imageId = uuidv4(); // Génère un ID unique pour l'image
    const s3Key = `${username}/${imageId}`; // Clé unique pour stocker l'image dans S3

    try {
        // Uploader le fichier dans S3
        const uploadParams = {
            Bucket: BUCKET_NAME,
            Key: s3Key,
            Body: file.buffer,
            ContentType: file.mimetype,
        };
        await s3.send(new PutObjectCommand(uploadParams));

        // Enregistrer les métadonnées de l'image dans DynamoDB
        await ddb.send(new PutCommand({
            TableName: IMAGES_TABLE,
            Item: {
                username,
                imageId,
                imageUrl: `https://${BUCKET_NAME}.s3.amazonaws.com/${s3Key}`,
                uploadedAt: new Date().toISOString(),
            },
        }));

        // Rediriger vers la page d'accueil
        res.redirect(`/album/home?username=${username}`);
    } catch (err) {
        console.error(err);
        res.status(500).json({ error: 'Erreur lors de l\'upload de l\'image' });
    }
});

module.exports = router;
